"""
vs_agent.py
An agent that use vector space Prehensions to understand its environment.
"""
import indra.grid_agent as ga
import indra.vector_space as vs
import indra.prehension_agent as pa


class VSAgent(pa.PrehensionAgent):
    """
    An agent that use Prehensions to understand its environment.
    """
    def __init__(self, name, goal, max_move=1, max_detect=1):
        super().__init__(name, goal, max_move, max_detect)
        self.my_filter = None
        self.stance = vs.VectorSpace()

    def survey_env(self):
        """
        Look around and see what surrounds us.
        """
        super().survey_env()
        other_pre = vs.VectorSpace()
        for other in self.neighbor_iter(view=self.my_view,
                                        filt_func=self.my_filter):
            # accumulate prehensions:
            other_pre = other.visible_stance().prehend(other_pre)
        return other_pre



# created based on C. Warrick's page here:
# https://chriswarrick.com/blog/2014/09/15/python-apps-the-right-way-entry_points-and-scripts/

from indra.entry_point import entry_point
import bigbox.big_box_run

models = {1: ("Big Box", bigbox.big_box_run.run)}


def main(args=None):
    entry_point(args, models)

if __name__ == "__main__":
    main()



"""
A script that runs big_box_model. It simulates the market economy
of consumers, mom and pops, and big boxes.
"""

import indra.utils as utils
import indra.prop_args as props
import bigbox.big_box_model as bb

# set up some file names:
MODEL_NM = "BigBoxModel"

def run():
    (prog_file, log_file, prop_file, results_file) = utils.gen_file_names(MODEL_NM)
    # We store basic parameters in a "property" file; this allows us to save
    #  multiple parameter sets, which is important in simulation work.
    #  We can read these in from file or set them here.
    pa = utils.read_props(MODEL_NM)
    if pa is None:
        pa = props.PropArgs(MODEL_NM, logfile=log_file, props=None)
        utils.get_grid_dims(pa, 50)
        utils.get_agent_num(pa, "num_consumers", "consumers",
                            bb.NUM_GOODS * 5 + 1)
        utils.get_agent_num(pa, "num_mom_and_pops", "mom and pop stores",
                            bb.NUM_GOODS)
        pa.ask("allowance", "What are the consumers' daily allowances?", 
               int, default=2)
    # right now, big box gets a multiple of m&p expenses and endowment
        pa.ask("endowment", "What are the small shops' initial endowments?", 
               int, default=30)
        pa.ask("expenses", "What are the small shops' expenses per period?",
               int, default=10)
        pa.ask("bb_start_period", "At what period should big boxes appear?",
               int, default=20, limits=(1, 100))
        pa.ask("pref_for_mp",
               "What's the consumer preference for mom and pops?",
               float, default=0.2, limits=(0.0, 1.0))
    # Now we create a town for our agents to act in:
    env = bb.EverytownUSA(pa.get("grid_width"),
                          pa.get("grid_height"),
                          model_nm=MODEL_NM,
                          props=pa)
    # Now we loop creating multiple agents with numbered names
    # based on the number of agents of that type to create:
    for i in range(pa.get("num_consumers")):
        env.add_agent(bb.Consumer("consumer" + str(i),
                                  goal=(i % bb.NUM_GOODS),
                                  init_state=0,
                                  allowance=pa.get("allowance")))
    for i in range(pa.get("num_mom_and_pops")):
        env.add_agent(bb.MomAndPop("mom_and_pop" + str(i),
                                   goal=(i % bb.NUM_GOODS),
                                   endowment=pa.get("endowment"),
                                   expenses=pa.get("expenses"),
                                   adj=pa.get("pref_for_mp")
                                   ))
    utils.run_model(env, prog_file, results_file)

if __name__ == "__main__":
    run()



from indra.entry_point import entry_point
import schelling.height_run
import schelling.auditorium_run
import schelling.segregation_run

models = {
    1: ("Segregation", schelling.segregation_run.run),
    2: ("Height", schelling.height_run.run),
    3: ("Auditorium", schelling.auditorium_run.run)
     }


def main(args=None):
    entry_point(args, models)

if __name__ == "__main__":
    main()



"""
grid_model.py
You can clone this file and its companion grid_run.py
to easily get started on a new grid model.
It also is a handy tool to have around for testing
new features added to the base system.
"""
import indra.grid_agent as ga


class TestGridAgent(ga.GridAgent):
    """
    An agent that prints its neighbors in preact
    and also jumps to an empty cell: defaut behavior
    from our ancestor.
    """

    def preact(self):
        (x, y) = self.pos
        print("With " + self.name
              + " we are looking around "
              + " x = " + str(x)
              + " y = " + str(y))
        print(self.name + " has neighbors: ")
        for neighbor in self.neighbor_iter():
            (x1, y1) = neighbor.pos
            print("    %i, %i" % (x1, y1))

    def postact(self):
        print("Agent %s postacting" % (self.name))



"""
basic_model.py
You can clone this file and its companion basic_run.py
to easily get started on a new model.
It also is a handy tool to have around
for testing new features added to the base system.
"""
import indra.entity as ent
import indra.env as env


class BasicAgent(ent.Agent):
    """
    An agent that just prints that it is around when asked to act
    """

    def __init__(self, name, goal):
        """
        A very basic init.
        """
        super().__init__(name, goal)

    def act(self):
        """
        Just print my name and goal!
        """
        # print("Agent %s with a goal of %s" % (self.name, self.goal))

    def postact(self):
        """
        Test of postact loop.
        """
        print("Agent %s is postacting" % (self.name))


class Gozer(BasicAgent):
    """
    A silly agent that destroys others, for demo purposes
    """

    def __init__(self):
        """
        Init Gozer with slightly different params.
        """
        super().__init__(name="Gozer the Destructor", goal="Destroy!")

    def postact(self):
        """
        Check to see if we have wiped everyone out.
        """
        e = self.env
        if len(e.agents) == 1:
            print("Gozer the Destructor has destroyed all!!")
        else:
            for agent in e.agents:
                if agent is not self:
                    e.agents.remove(agent)
                    print("Gozer has destroyed "
                          + agent.name + "!")
                    return


class BasicEnv(env.Environment):
    """
    This environment doesn't really do anything.
    """

    def __init__(self, model_nm=None, props=None):
        super().__init__("Basic environment",
                         preact=True,
                         postact=True,
                         model_nm=model_nm,
                         props=props)

    def preact_loop(self):
        print("Preact loop: demonstrating backwards looping")
        for agent in reversed(self.agents):
            print("Agent: " + agent.name)



"""
Filename: entity.py
Author: Gene Callahan and Brandon Logan
This module contains the base classes for agent-based modeling in Indra.
"""

# import logging
from abc import abstractmethod
import indra.node as node


class Entity(node.Node):
    """
    This is the base class of all agents, environments,
    and objects contained in an environment.
    
    Attribute:
        env: the environment the agent lives within
    """

    def __init__(self, name):
        super().__init__(name)
        self.env = None

    def add_env(self, env):
        """
        Note our environment.
        """
        self.env = env


class Agent(Entity):

    """
    This class adds a goal to an entity
    and is the base for all other agents.
    
    All agents act. For reasons of clarity
    they *may* also act before or after this
    action.
    
    Attribute:
        goal: Often, this is a string which helps
            the user understand what the agent
            is "up to." It might also be used to
            inform the env what kind of action
            the agent will take.
    """

    def __init__(self, name, goal=None):
        super().__init__(name)
        self.goal = goal

    @abstractmethod
    def act(self):
        """
        What agents do.
        """
        pass

    def preact(self):
        """
        What agents do before they act.
        """
        pass

    def postact(self):
        """
        What agents do after they act.
        """
        pass

    def survey_env(self, universal):
        """
        Empty for the moment.
        """
        return None

    def debug_info(self):
        """
        Relevant debugging info.
        """
        s = super().debug_info() + "\ngoal: " + str(self.goal)
        return s

    def to_json(self):
        """
        We're going to make a dictionary of the 'safe' parts of the object to
        output to a json file. (We can't output the env, for instance, since
        IT contains a reference to each agent!)
        """
        safe_fields = super().to_json()
        safe_fields["goal"] = self.goal
        return safe_fields
